@@ -18,10 +18,12 @@
  **********************************************************************/
 
 #include          "mfcpch.h"     //precompiled headers
+#include          <stdlib.h>
+#include          "ndminx.h"
 #include          "serialis.h"
 #include          "points.h"
 
-ELISTIZE_S (ICOORDELT)           //turn to list
+ELISTIZE (ICOORDELT)           //turn to list
 bool FCOORD::normalise() {  //Convert to unit vec
   float len = length ();
 
@@ -33,6 +35,17 @@ bool FCOORD::normalise() {  //Convert to unit vec
   return true;
 }
 
+// Set from the given x,y, shrinking the vector to fit if needed.
+void ICOORD::set_with_shrink(int x, int y) {
+  // Fit the vector into an ICOORD, which is 16 bit.
+  int factor = 1;
+  int max_extent = MAX(abs(x), abs(y));
+  if (max_extent > MAX_INT16)
+    factor = max_extent / MAX_INT16 + 1;
+  xcoord = x / factor;
+  ycoord = y / factor;
+}
+
 // The fortran/basic sgn function returns -1, 0, 1 if x < 0, x == 0, x > 0
 // respectively.
 static int sign(int x) {
@@ -49,7 +62,7 @@ static int sign(int x) {
 // subtract major and step a minor step.
 
 void ICOORD::setup_render(ICOORD* major_step, ICOORD* minor_step,
-                          int* major, int* minor) {
+                          int* major, int* minor) const {
   int abs_x = abs(xcoord);
   int abs_y = abs(ycoord);
   if (abs_x >= abs_y) {
@@ -70,33 +83,3 @@ void ICOORD::setup_render(ICOORD* major_step, ICOORD* minor_step,
     *minor = abs_x;
   }
 }
-
-
-void ICOORD::serialise_asc(         //convert to ascii
-                           FILE *f  //file to write
-                          ) {
-  serialise_INT32(f, xcoord);
-  serialise_INT32(f, ycoord);
-}
-
-
-void ICOORD::de_serialise_asc(         //convert from ascii
-                              FILE *f  //file to write
-                             ) {
-  xcoord = (inT16) de_serialise_INT32 (f);
-  ycoord = (inT16) de_serialise_INT32 (f);
-}
-
-
-void ICOORDELT::serialise_asc(         //convert to ascii
-                              FILE *f  //file to write
-                             ) {
-  ((ICOORD *) this)->serialise_asc (f);
-}
-
-
-void ICOORDELT::de_serialise_asc(         //convert from ascii
-                                 FILE *f  //file to write
-                                ) {
-  ((ICOORD *) this)->de_serialise_asc (f);
-}
