@@ -5,10 +5,10 @@
 // For Unix and mac the file does nothing. It needs to be included in all cpp
 // files for compatibility with the PC pre-compiled header mechanism.
 #ifdef __MSW32__
-#ifdef __IPEREGDLL
+#ifndef _AFXDLL
 #define WIN32_LEAN_AND_MEAN
 #define STRICT 1
-#include <windows.h>
+//#include <windows.h>
 #include <stdlib.h>
 #else
 #define VC_EXTRALEAN             // Exclude rarely-used stuff from Windows headers
