@@ -19,12 +19,16 @@
           Include Files and Type Defines
 ----------------------------------------------------------------------------**/
 #include "cutoffs.h"
+
+#include <stdio.h>
+
+#include "classify.h"
 #include "efio.h"
+#include "globals.h"
+#include "helpers.h"
 #include "scanutils.h"
 #include "serialis.h"
 #include "unichar.h"
-#include "globals.h"
-#include <stdio.h>
 
 #define REALLY_QUOTE_IT(x) QUOTE_IT(x)
 
@@ -34,40 +38,41 @@
               Public Code
 ----------------------------------------------------------------------------**/
 /*---------------------------------------------------------------------------*/
-void ReadNewCutoffs(const char *Filename,
-                    CLASS_TO_INDEX ClassMapper,
-                    CLASS_CUTOFF_ARRAY Cutoffs) {
+namespace tesseract {
+void Classify::ReadNewCutoffs(FILE *CutoffFile, inT64 end_offset,
+                              CLASS_CUTOFF_ARRAY Cutoffs) {
 /*
  **	Parameters:
  **		Filename	name of file containing cutoff definitions
- **             ClassMapper     array which maps class id's to class indexes
  **		Cutoffs		array to put cutoffs into
  **	Globals: none
  **	Operation: Open Filename, read in all of the class-id/cutoff pairs
  **		and insert them into the Cutoffs array.  Cutoffs are
- **		inserted in the array so that the array is indexed by
- **		class index rather than class id.  Unused entries in the
+ **		indexed in the array by class id.  Unused entries in the
  **		array are set to an arbitrarily high cutoff value.
  **	Return: none
  **	Exceptions: none
  **	History: Wed Feb 20 09:38:26 1991, DSJ, Created.
  */
-  FILE *CutoffFile;
   char Class[UNICHAR_LEN + 1];
   CLASS_ID ClassId;
   int Cutoff;
   int i;
 
-  CutoffFile = Efopen (Filename, "r");
-
   for (i = 0; i < MAX_NUM_CLASSES; i++)
     Cutoffs[i] = MAX_CUTOFF;
 
-  while (fscanf (CutoffFile, "%" REALLY_QUOTE_IT(UNICHAR_LEN) "s %d",
-                 Class, &Cutoff) == 2) {
-    ClassId = unicharset.unichar_to_id(Class);
-    Cutoffs[ClassMapper[ClassId]] = Cutoff;
+  while ((end_offset < 0 || ftell(CutoffFile) < end_offset) &&
+         fscanf(CutoffFile, "%" REALLY_QUOTE_IT(UNICHAR_LEN) "s %d",
+                Class, &Cutoff) == 2) {
+    if (strcmp(Class, "NULL") == 0) {
+      ClassId = unicharset.unichar_to_id(" ");
+    } else {
+      ClassId = unicharset.unichar_to_id(Class);
+    }
+    Cutoffs[ClassId] = Cutoff;
+    SkipNewline(CutoffFile);
   }
-  fclose(CutoffFile);
-
 }                                /* ReadNewCutoffs */
+
+}  // namespace tesseract
